@@ -1,7 +1,17 @@
-from .nvml import NVMLReader
+from .nvml import NVMLReader, DataThroughput
 from .rapl import RAPLReader
 from .base import BaseReader
-from .utils import Quantity, Energy, Power, Temperature, Unit, Joule, Watt, Celsius
+from .utils import (
+    Quantity,
+    Energy,
+    Power,
+    Temperature,
+    Unit,
+    Joule,
+    Watt,
+    Celsius,
+    Utilization,
+)
 
 __all__ = [
     "NVMLReader",
@@ -15,4 +25,6 @@ __all__ = [
     "Joule",
     "Watt",
     "Celsius",
+    "Utilization",
+    "DataThroughput",
 ]
