@@ -8,36 +8,49 @@
 
 # Usage function to display help
 usage() {
-    echo "Usage: $0 (ID) [filepath]"
-    exit 1
+    echo "Usage: $0 [-q] [-f filepath] ID"
+    echo "  -q          Quiet mode (suppress output)"
+    echo "  -f filepath Optional file path to monitor"
+    echo "  ID          Run ID to wait for"
+    exit "${1:-0}"
 }
 
-# Get the ID from the command line argument
+# Parse options
+QUIET=false
+FILEPATH=""
+while getopts "qf:h" opt; do
+    case "$opt" in
+        q) QUIET=true ;;
+        f) FILEPATH="$OPTARG" ;;
+        h) usage 0 ;;
+        *) usage 1 ;;
+    esac
+done
+shift $((OPTIND - 1))
+
+# Get the ID from the remaining arguments
 if [ $# -ge 1 ]; then
     ID="$1"
 else
     usage
 fi
 
-# Check if an input filename was given
-if [ $# -ge 2 ]; then
-    FILEPATH="$2"
-else
+# If no filepath was provided via -f flag, generate it
+if [ -z "$FILEPATH" ]; then
     # Get the WattAMeter powerlog file path for the current node
     NODE=$(hostname)
     FILEPATH=$(wattameter_powerlog_filename --suffix "${ID}-${NODE}")
-    echo "Waiting for ${FILEPATH} to be ready for run ID ${ID}..."
 fi
 
-# Wait for the file to be created
-until [ -f "${FILEPATH}" ]; do
-    sleep 1  # Wait for 1 second before checking again
-done
-
 # Wait until ID can be read from the file
-until grep -q "run $ID" "${FILEPATH}"; do
+if [ "$QUIET" = false ]; then
+    echo "Waiting for ${FILEPATH} to be ready for run ID ${ID}..."
+fi
+until grep -qs "run $ID" "${FILEPATH}"; do
     sleep 1  # Wait for 1 second before checking again
 done
 
-echo "${FILEPATH} is ready for run ID ${ID}."
+if [ "$QUIET" = false ]; then
+    echo "${FILEPATH} is ready for run ID ${ID}."
+fi
 exit 0
\ No newline at end of file
